@@ -26,25 +26,14 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "taskgroupingproxymodel.h"
 #include "tasktools.h"
 
-#include <config-X11.h>
-
 #include "launchertasksmodel.h"
-#include "waylandtasksmodel.h"
 #include "startuptasksmodel.h"
-#if HAVE_X11
-#include "xwindowtasksmodel.h"
-#endif
-
-#include <KWindowSystem>
+#include "windowtasksmodel.h"
 
 #include <QGuiApplication>
 #include <QTimer>
 #include <QUrl>
 
-#if HAVE_X11
-#include <QX11Info>
-#endif
-
 #include <numeric>
 
 namespace TaskManager
@@ -58,7 +47,7 @@ public:
 
     static int instanceCount;
 
-    static AbstractTasksModel* windowTasksModel;
+    static WindowTasksModel* windowTasksModel;
     static StartupTasksModel* startupTasksModel;
     LauncherTasksModel* launcherTasksModel = nullptr;
     ConcatenateTasksProxyModel* concatProxyModel = nullptr;
@@ -89,6 +78,7 @@ public:
     int groupingWindowTasksThreshold = -1;
 
     void initModels();
+    void initLauncherTasksModel();
     void updateAnyTaskDemandsAttention();
     void updateManualSortMap();
     void consolidateManualSortMapForGroup(const QModelIndex &groupingProxyIndex);
@@ -123,7 +113,7 @@ private:
 };
 
 int TasksModel::Private::instanceCount = 0;
-AbstractTasksModel* TasksModel::Private::windowTasksModel = nullptr;
+WindowTasksModel* TasksModel::Private::windowTasksModel = nullptr;
 StartupTasksModel* TasksModel::Private::startupTasksModel = nullptr;
 ActivityInfo* TasksModel::Private::activityInfo = nullptr;
 int TasksModel::Private::activityInfoUsers = 0;
@@ -155,23 +145,17 @@ TasksModel::Private::~Private()
 void TasksModel::Private::initModels()
 {
     // NOTE: Overview over the entire model chain assembled here:
-    // {X11,Wayland}WindowTasksModel, StartupTasksModel, LauncherTasksModel
+    // WindowTasksModel, StartupTasksModel, LauncherTasksModel
     //  -> concatProxyModel concatenates them into a single list.
     //   -> filterProxyModel filters by state (e.g. virtual desktop).
     //    -> groupingProxyModel groups by application (we go from flat list to tree).
     //     -> flattenGroupsProxyModel (optionally, if groupInline == true) flattens groups out.
     //      -> TasksModel collapses (top-level) items into task lifecycle abstraction; sorts.
 
-    if (!windowTasksModel && QGuiApplication::platformName().startsWith(QLatin1String("wayland"))) {
-        windowTasksModel = new WaylandTasksModel();
+    if (!windowTasksModel) {
+        windowTasksModel = new WindowTasksModel();
     }
 
-#if HAVE_X11
-    if (!windowTasksModel && QX11Info::isPlatformX11()) {
-        windowTasksModel = new XWindowTasksModel();
-    }
-#endif
-
     QObject::connect(windowTasksModel, &QAbstractItemModel::rowsInserted, q,
         [this]() {
             if (sortMode == SortActivity) {
@@ -208,17 +192,10 @@ void TasksModel::Private::initModels()
         startupTasksModel = new StartupTasksModel();
     }
 
-    launcherTasksModel = new LauncherTasksModel(q);
-    QObject::connect(launcherTasksModel, &LauncherTasksModel::launcherListChanged,
-        q, &TasksModel::launcherListChanged);
-    QObject::connect(launcherTasksModel, &LauncherTasksModel::launcherListChanged,
-        q, &TasksModel::updateLauncherCount);
-
     concatProxyModel = new ConcatenateTasksProxyModel(q);
 
     concatProxyModel->addSourceModel(windowTasksModel);
     concatProxyModel->addSourceModel(startupTasksModel);
-    concatProxyModel->addSourceModel(launcherTasksModel);
 
     // If we're in manual sort mode, we need to seed the sort map on pending row
     // insertions.
@@ -294,8 +271,8 @@ void TasksModel::Private::initModels()
     filterProxyModel->setSourceModel(concatProxyModel);
     QObject::connect(filterProxyModel, &TaskFilterProxyModel::virtualDesktopChanged,
         q, &TasksModel::virtualDesktopChanged);
-    QObject::connect(filterProxyModel, &TaskFilterProxyModel::screenChanged,
-        q, &TasksModel::screenChanged);
+    QObject::connect(filterProxyModel, &TaskFilterProxyModel::screenGeometryChanged,
+        q, &TasksModel::screenGeometryChanged);
     QObject::connect(filterProxyModel, &TaskFilterProxyModel::activityChanged,
         q, &TasksModel::activityChanged);
     QObject::connect(filterProxyModel, &TaskFilterProxyModel::filterByVirtualDesktopChanged,
@@ -388,7 +365,8 @@ void TasksModel::Private::initModels()
 
                 // When a window or startup task is removed, we have to trigger a re-filter of
                 // matching launchers to (possibly) pop them back in.
-                if (!(sourceIndex.data(AbstractTasksModel::IsWindow).toBool()
+                if (!launcherTasksModel
+                      || !(sourceIndex.data(AbstractTasksModel::IsWindow).toBool()
                       || sourceIndex.data(AbstractTasksModel::IsStartup).toBool())) {
                     continue;
                 }
@@ -455,6 +433,21 @@ void TasksModel::Private::updateAnyTaskDemandsAttention()
     }
 }
 
+void TasksModel::Private::initLauncherTasksModel()
+{
+    if (launcherTasksModel) {
+        return;
+    }
+
+    launcherTasksModel = new LauncherTasksModel(q);
+    QObject::connect(launcherTasksModel, &LauncherTasksModel::launcherListChanged,
+        q, &TasksModel::launcherListChanged);
+    QObject::connect(launcherTasksModel, &LauncherTasksModel::launcherListChanged,
+        q, &TasksModel::updateLauncherCount);
+
+    concatProxyModel->addSourceModel(launcherTasksModel);
+}
+
 void TasksModel::Private::updateManualSortMap()
 {
     // Empty map; full sort.
@@ -846,6 +839,10 @@ QVariant TasksModel::data(const QModelIndex &proxyIndex, int role) const
 
 void TasksModel::updateLauncherCount()
 {
+    if (!d->launcherTasksModel) {
+        return;
+    }
+
     int count = 0;
 
     for (int i = 0; i < rowCount(); ++i) {
@@ -880,14 +877,14 @@ void TasksModel::setVirtualDesktop(int virtualDesktop)
     d->filterProxyModel->setVirtualDesktop(virtualDesktop);
 }
 
-int TasksModel::screen() const
+QRect TasksModel::screenGeometry() const
 {
-    return d->filterProxyModel->screen();
+    return d->filterProxyModel->screenGeometry();
 }
 
-void TasksModel::setScreen(int screen)
+void TasksModel::setScreenGeometry(const QRect &geometry)
 {
-    d->filterProxyModel->setScreen(screen);
+    d->filterProxyModel->setScreenGeometry(geometry);
 }
 
 QString TasksModel::activity() const
@@ -1114,29 +1111,26 @@ QStringList TasksModel::launcherList() const
 
 void TasksModel::setLauncherList(const QStringList &launchers)
 {
-    if (d->launcherTasksModel) {
-        d->launcherTasksModel->setLauncherList(launchers);
-        d->launchersEverSet = true;
-    }
+    d->initLauncherTasksModel();
+    d->launcherTasksModel->setLauncherList(launchers);
+    d->launchersEverSet = true;
 }
 
 bool TasksModel::requestAddLauncher(const QUrl &url)
 {
-    if (d->launcherTasksModel) {
-        bool added = d->launcherTasksModel->requestAddLauncher(url);
+    d->initLauncherTasksModel();
 
-        // If using manual and launch-in-place sorting with separate launchers,
-        // we need to trigger a sort map update to move any window tasks to
-        // their launcher position now.
-        if (added && d->sortMode == SortManual && (d->launchInPlace || !d->separateLaunchers)) {
-            d->updateManualSortMap();
-            d->forceResort();
-        }
+    bool added = d->launcherTasksModel->requestAddLauncher(url);
 
-        return added;
+    // If using manual and launch-in-place sorting with separate launchers,
+    // we need to trigger a sort map update to move any window tasks to
+    // their launcher position now.
+    if (added && d->sortMode == SortManual && (d->launchInPlace || !d->separateLaunchers)) {
+        d->updateManualSortMap();
+        d->forceResort();
     }
 
-    return false;
+    return added;
 }
 
 bool TasksModel::requestRemoveLauncher(const QUrl &url)
@@ -1181,6 +1175,13 @@ void TasksModel::requestNewInstance(const QModelIndex &index)
     }
 }
 
+void TasksModel::requestOpenUrls(const QModelIndex &index, const QList<QUrl> &urls)
+{
+    if (index.isValid() && index.model() == this) {
+        d->abstractTasksSourceModel->requestOpenUrls(mapToSource(index), urls);
+    }
+}
+
 void TasksModel::requestClose(const QModelIndex &index)
 {
     if (index.isValid() && index.model() == this) {
@@ -1251,6 +1252,13 @@ void TasksModel::requestVirtualDesktop(const QModelIndex &index, qint32 desktop)
     }
 }
 
+void TasksModel::requestActivities(const QModelIndex &index, const QStringList &activities)
+{
+    if (index.isValid() && index.model() == this) {
+        d->groupingProxyModel->requestActivities(mapToSource(index), activities);
+    }
+}
+
 void TasksModel::requestPublishDelegateGeometry(const QModelIndex &index, const QRect &geometry, QObject *delegate)
 {
     if (index.isValid() && index.model() == this) {
@@ -1295,7 +1303,8 @@ bool TasksModel::move(int row, int newPos)
     }
 
     if (d->separateLaunchers) {
-        const int firstTask = (d->launchInPlace ? d->launcherTasksModel->rowCount() : launcherCount());
+        const int firstTask = (d->launcherTasksModel ?
+            (d->launchInPlace ? d->launcherTasksModel->rowCount() : launcherCount()) : 0);
 
         // Don't allow launchers to be moved past the last launcher.
         if (isLauncherMove && newPos >= firstTask) {
@@ -1428,8 +1437,9 @@ void TasksModel::syncLaunchers()
 
     QMap<int, QUrl> sortedLaunchers;
 
-    foreach(const QUrl &launcherUrl, launcherList()) {
+    foreach(const QString &launcherUrlStr, launcherList()) {
         int row = -1;
+        QUrl launcherUrl(launcherUrlStr);
 
         for (int i = 0; i < rowCount(); ++i) {
             const QUrl &rowLauncherUrl = index(i, 0).data(AbstractTasksModel::LauncherUrlWithoutIcon).toUrl();
